%% -------------------------------------------------------
%
%    plotSetupOverview - Plots a Mesh also calculates the face centroids as
%                        a by-product
%
%    Ver. 1.0
%
%    Created:           Jan Kost (02.06.2018)
%    Last modified:     Jan Kost (20.10.2020)
%
%    Institute of Biomedical Engineering
%    Karlsruhe Institute of Technology
%
%    http://www.ibt.kit.edu
%
%    Copyright 2020 - All rights reserved.
%
% ------------------------------------------------------
%
% [plotHandle,centroids] = patchPlotFaces(faces,vertices,figTitle,...
%                           faceColor,dispPlot,plotNormals,normals,showIDs) 
% 
%        input: 
%               faces:      m x 3 matrix, where m is the number of faces in
%                           the mesh and each row contains the IDs of the
%                           three vertices belonging to the corresponding
%                           face.
%
%               vertices:   n x 3 matrix, where n is the number of vertices
%                           in the mesh and each row contains the cartesian
%                           coordinates of a vertex.
%
%               figTitle    The title of the figure window.
%
%               faceColor:  Sets the color of the faces in the plot.
%
%               dispPlot:   Toggles figure visibility.
%
%               merge:      Set if the function should plot into an
%                           existing figure.
%
%               handle:     Handle of the figure to plot into (can be an
%                           empty figure (figure('Visible','off')) if 
%                           "merge" is false)
%
%               plotNormals: Set if the function should also plot the
%                            normals of the surface primitives
%
%               normals:    mx3 Matrix: Normals of the faces. (Can be 
%                           an empty array if "plotNormals" is false)
%
%        output: 
%               plotHandle: The figure handle generated by the function.
%
%               centroids:  m x 3 matrix, where m is the number of faces in
%                           the mesh and each row contains the cartesian 
%                           coordinates of the corresponding face's 
%                           centroid.
%
function [plotHandle,centroids] = patchPlotFaces(faces,vertices,...
                   figTitle,faceColor,dispPlot,plotNormals,normals,showIDs)     
    %calculate face centroids 
    tmpCentroids = zeros((size(faces,1)),3);%preallocate
    x = vertices(:,1);
    y = vertices(:,2);
    z = vertices(:,3);
    for i = 1:size(faces,1) % for each face
        v_1 = faces(i,1);
        v_2 = faces(i,2);
        v_3 = faces(i,3);
        %calculate face cetroids
        c = [(x(v_1)+x(v_2)+x(v_3))/3,...
             (y(v_1)+y(v_2)+y(v_3))/3,...
             (z(v_1)+z(v_2)+z(v_3))/3]; 
        tmpCentroids(i,:) = c;
    end
    %plot the mesh
    fig1 = figure('Name',figTitle,'NumberTitle','off','Visible',dispPlot);
    patch('Faces',faces,'Vertices',vertices, 'FaceColor',faceColor);
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    if showIDs
        hold on
        %plot face IDs
        for j = 1:size(faces,1)
            shift = 0.02;
            x_t = tmpCentroids(j,1);
            s_x = shift*normals(j,1);
            y_t = tmpCentroids(j,2);
            s_y = shift*normals(j,2);
            z_t = tmpCentroids(j,3);
            s_z = shift*normals(j,3);
            text(s_x+x_t,s_y+y_t,s_z+z_t,num2str(j))
        end
    end
    if plotNormals 
        hold on
        %plot the face normals
        quiver3(tmpCentroids(:,1),tmpCentroids(:,2),tmpCentroids(:,3),normals(:,1),...
                                                normals(:,2),normals(:,3));
    end
    hold off
    plotHandle = fig1;
    centroids = tmpCentroids;
end